use poise::ChoiceParameter;
use radiobrowser::ApiStation;
use serenity::{all::CreateEmbed, async_trait};

use crate::utils::utilities::hash_to_u32;

#[derive(ChoiceParameter, Debug)]
pub enum NumberOfEntries {
    #[name = "5"]
    Five = 5,
    #[name = "10"]
    Ten = 10,
    #[name = "15"]
    Fifteen = 15
}

#[async_trait]
pub trait GenerateEmbed {
    async fn generate_embed(&self) -> CreateEmbed;
}

#[allow(dead_code)]
pub struct SpotifyInfo {
    pub name: String,
    pub interpret: String,
    // client
    pub duration: String,
}

#[allow(dead_code)]
pub struct GeneralInfo {
    pub name: String,
    pub site: String,
    // client
    pub duration: Option<String>
}


#[async_trait]
impl GenerateEmbed for GeneralInfo {
    async fn generate_embed(&self) -> CreateEmbed {
        CreateEmbed::new()
    }
}

#[async_trait]
impl GenerateEmbed for SpotifyInfo {
    async fn generate_embed(&self) -> CreateEmbed {
        CreateEmbed::new()
    }
}

#[async_trait]
impl GenerateEmbed for ApiStation {
    async fn generate_embed(&self) -> CreateEmbed {
        let embed= CreateEmbed::new()
            .title("Current station:")
            .thumbnail(&self.favicon)
            .field("Name:", &self.name, true)
            .field("Country:", &self.country, true)
            .field("Language:", &self.language, true)
            .field("Website:", &self.homepage, true)
            .field("Stream:", &self.url, true)
            .field("Tags:", &self.tags, false)
            .color(hash_to_u32(&self.stationuuid));

        embed
    }
}

// pub struct PlayingInfo<T> {
//     pub data: T,
//     pub create_embed: Box<dyn Fn(T) -> CreateEmbed + Send + Sync>
// }

#[allow(dead_code)]
pub enum PlayingInfoType {
    SpotifyInfo(SpotifyInfo),
    RadioInfo(ApiStation),
    GeneralInfo(GeneralInfo)
}

impl PlayingInfoType {
    pub async fn generate_embed(&self) -> CreateEmbed {
        match self {
            PlayingInfoType::SpotifyInfo(v) => v.generate_embed().await,
            PlayingInfoType::RadioInfo(v) => v.generate_embed().await,
            PlayingInfoType::GeneralInfo(v) => v.generate_embed().await
        }
    }
}
