use std::vec;
use std::env;

use songbird::input::YoutubeDl;

use crate::commands::util::connect;
use crate::util::poise_context_extension::ContextExt;
use crate::types::{Context, Error};
use crate::commands::voice::util::autocomplete_channel;

// TODO: search, queue
#[poise::command(
    slash_command,
    description_localized("en-US", "Plays music from YouTube URL")
)]
pub async fn play_yt(ctx: Context<'_>,
    #[autocomplete = "autocomplete_channel"]
    #[description = "Voice channel name: "]
    channel: Option<String>,
    #[description = "Source URL: "]
    url: String,
) -> Result<(), Error> {

    if ctx.guild().is_none() {
        ctx.reply_ephemeral("Can't use this outside of guild").await?;
        return Ok(());
    }

    let manager = songbird::get(ctx.serenity_context())
        .await
        .expect("Songbird Voice client placed in at initialisation.")
        .clone();

    let Some(guild_id) = ctx.guild_id() else {
        ctx.reply_ephemeral("Guild id not found").await?;
        return Ok(())
    };

    let http_client = ctx.data().http_client.clone();

    if manager.get(guild_id).is_none() {
        match connect(&ctx, guild_id, channel).await {
            Ok(_) => (),
            Err(e) => {
                ctx.reply_ephemeral(&e.to_string()).await?;
                return Ok(())
            }
        }
    }

    if let Some(handler_lock) = manager.get(guild_id) {
        let mut handler = handler_lock.lock().await;

        let cookies_path = match env::var("COOKIES") {
            Ok(path) => path,
            Err(e) => {
                let _ = ctx.reply_ephemeral("There was an error (did not find cookies)").await;
                dbg!(e);
                return Ok(());
            }
        };

        let po_token = match env::var("PO_TOKEN") {
            Ok(token) => token,
            Err(e) => {
                let _ = ctx.reply_ephemeral("There was an error (did not find token)").await;
                dbg!(e);
                return Ok(());
            }
        };

        let arguments: Vec<String> = vec![
            "--extractor-args".into(),
            format!("youtube:player-client=web,default;po_token=web+{po_token}"),
            "--cookies".into(),
            cookies_path
        ];
        let src = YoutubeDl::new(http_client, url).user_args(arguments);
        handler.enqueue_input(src.into()).await;
    }
    else {
        ctx.reply_ephemeral("Not in a voice channel").await?;
        return Ok(())
    }

    ctx.reply_ephemeral("Done!").await?;
    Ok(())
}
